#!/bin/bash

DIR=`dirname "$0"`
if [ ! -e "${DIR}/openssl.cnf" ]||[ ! -e "${DIR}/cert.conf" ]; then
  echo "Chyba subor openssl.cnf alebo cert.conf v $DIR"
  exit 1
fi

. "${DIR}/cert.conf"
CNF="$DIR/openssl.cnf"
export CA_DIR="$DIR/ca"

function generuj()
{
  BASENAME="${1}/cert"
  EXT="${2}"
  if [ ! -f "$BASENAME.key" ]; then
    openssl req -nodes -new -x509 -keyout "$BASENAME.key" -out "$BASENAME.key" -subj "/C=${C_C}/ST=${C_ST}/L=${C_L}/O=${C_O}/OU=${C_OU}/CN=${3}" -days 3650 -config "$CNF"
  else
    echo Kluc $BASENAME uz bol vygenerovany, takze ignorujem jeho tvorbu a idem na dalsi krok...
  fi
  if [ ! -f "$BASENAME.key" ]; then
    echo Kluc $BASENAME nebol vytvoreny, koncim!
    exit 1
  fi

  TMPFILE="${1}/tmp.pem"

  rm -f "$TMPFILE"

  openssl x509 -x509toreq -in "$BASENAME.key" -signkey "$BASENAME.key" -out "$TMPFILE"
  if [ ! -f "$TMPFILE" ]; then
    echo Nepodarilo sa vytvorit x509 poziadavku pre CA z kluca $BASENAME, koncim!
    exit 2
  fi

  if [ ! -f "$BASENAME.crt" ]; then
    openssl ca -config "$CNF" -extensions "$EXT" -policy policy_anything -key "$KEY" -out "$BASENAME.crt" -infiles "$TMPFILE"
  else
    echo Kluc $BASENAME uz bol podpisany CA, takze ignorujem podpisovanie a idem na dalsi krok...
  fi

  rm -f "$TMPFILE"

  if [ ! -f "$BASENAME.crt" ]; then
    echo Kluc $BASENAME nebol uspesne podpisany!!!
    exit 3
  fi
}

if [ -z "$1" ]|| [ -z "$2" ]; then
  echo "Treba zadat typ a nazov, napriklad:"
  echo "$0 typ bowling"
  exit 1
fi

TYP="$1"
if [ "$TYP" != "client" ] && [ "$TYP" != "server" ]; then
  echo "Typ musi byt client alebo server!"
  exit 2
fi

ADIR="$DIR/$TYP/$2"
mkdir -p "${ADIR}" >/dev/null 2>/dev/null
if [ -d "${ADIR}" ]; then
  if [ "$TYP" = "server" ]; then
    generuj "${ADIR}" "$TYP" "${2}${SERVER_SUFFIX}.${C_OU}"
    if [ "$SERVER_DH" = "yes" ]; then
      openssl dhparam -out "${ADIR}/dh2048.pem" 2048
    fi
  fi
  if [ "$TYP" = "client" ]; then
    generuj "${ADIR}" "$TYP" "${2}${CLIENT_SUFFIX}.${C_OU}"
  fi
  echo Zjavne sa vsetko podarilo
else
  echo Neviem vytvorit adresar
fi
